#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "--" "$@"
########################################################################
#
#                               tkMemoPad
#                              version 1.1.0
#
########################################################################

# Default Preferences

set pref_lang jpn;                   # Language(jpn or eng)
set pref_save_timesec 20;            # auto save timer (sec) if 0 then don't use save timer
set pref_quitAlert 0;                # when quittig,show alert(0 or 1)
set pref_sessionfile {tkMemoPad.session};   # session file name
set pref_rcfile      {tkMemoPad.resource};  # resource file name
set pref_datadir {./.tkMemoPad};     # working directory
set pref_win_name {tkMemoPad};       # this apprications name
set pref_tabstop 4;                  #tabstop
set pref_shell [file normalize $::env(ComSpec)]

# Change Pref by platform and language
set pref_clockformat {%B %d, %Y  %H:%M};  # US Format(if lang is jpn, redefine to Japanease Format in 26line)
font create font_note -size 10

if { $pref_lang=={jpn} } {
	set pref_clockformat  "%Y\u5E74%m\u6708%d\u65E5(%J) %H:%M"; #Japanease Format
	if { $tcl_platform(platform)=={windows} } {
		font configure font_note -family {MS Gothic}
	} else {
		font configure font_note -size {12} 
	}
}

set pref_col_text_bg  #555566;  # background color
set pref_col_text_fg  #FFFFFF;  # foreground color
set pref_col_text_sbg #DDDDDD;  # select background color
set pref_col_text_sfg #000000;  # select foreground color
set pref_col_text_cur #00FF00;  # cursor color
set pref_col_page_bg  #222233;  # pagenumber background color
set pref_col_page_fg  #FFFFFF;  # pagenumber foreground color
set pref_col_btn_bg   #222233;  # button background color
set pref_col_btn_fg   #FFFFFF;  # button foreground color

# Don't Edit Below 
# =====================================================================
set pref_win_geometry 180x200+20+20; # window geometry [w]x[h]+[x]+[y]
set pref_win_topmost 0;              # window always on top(0 or 1)
set pref_page 1;                     # page index number
set pref_wrap 1;                     # wrap (0 or 1)

array set slaveinfo {}
array set summary {}
array set jumplist_sort {
	order   -increasing
	key	    0
	type    -dictionary
}

set WEEKDAY_JPN [list \u65E5 \u6708 \u706B \u6C34 \u6728 \u91D1 \u571F]
array set UISTRING {
	jpn ::UISTRING_JPN
	eng ::UISTRING_ENG
}
array set UISTRING_JPN {
	cut				\u5207\u308A\u53D6\u308A
	copy			\u30B3\u30D4\u30FC
	paste			\u8CBC\u308A\u4ED8\u3051
	selectall		\u5168\u3066\u9078\u629E
	back			\u524D\u30DA\u30FC\u30B8\u3078
	next			\u6B21\u30DA\u30FC\u30B8\u3078
	exchange_back	\u524D\u30DA\u30FC\u30B8\u3068\u4EA4\u63DB
	exchange_next	\u6B21\u30DA\u30FC\u30B8\u3068\u4EA4\u63DB
	go				\u30DA\u30FC\u30B8\u30B8\u30E3\u30F3\u30D7\u306E\u8868\u793A/\u66F4\u65B0...
	insert_date		\u65E5\u4ED8\u3092\u633F\u5165
	count_char		\u6587\u5B57\u6570\u30AB\u30A6\u30F3\u30C8
	countnum_all	\u6587\u5B57\u6570
	countnum_char	\u6587\u5B57\u6570\uFF08\u6539\u884C\u9664\u304F\uFF09
	countnum_line	\u884C\u6570
	eval			Tcl/Tk\u3067\u5B9F\u884C
	exec			Shell\u3067\u5B9F\u884C
	del_intrp		\u30A4\u30F3\u30BF\u30D7\u30EA\u30BF\u306E\u505C\u6B62...
	wrap			\u6298\u308A\u8FD4\u3057
	topmost			\u5E38\u306B\u524D\u9762
	quit			\u7D42\u4E86
	msg_stop_intrp	\u505C\u6B62\u3057\u305F\u3044\u30A4\u30F3\u30BF\u30D7\u30EA\u30BF\u3092\u9078\u629E\u3057\u3066\u304F\u3060\u3055\u3044
	msg_jump_page   \u30DA\u30FC\u30B8\u756A\u53F7\u3092\u6307\u5B9A\u3057\u3066\u304F\u3060\u3055\u3044
	sort_asc		\u6607\u9806\u306B
	sort_desc		\u964D\u9806\u306B
	sort_page		\u30DA\u30FC\u30B8\u756A\u53F7
	sort_sum		\u8981\u7D04
	sort_dic		\u8F9E\u66F8\u9806
}
array set UISTRING_ENG {
	cut				Cut
	copy			Copy
	paste			Paste
	selectall		{Select All}
	back			{Go Back}
	next			{Go Next}
	exchange_back	{Exchange Cur for previous}
	exchange_next	{Exchange Cur for next}
	go              {Show/Update PageJump...}
	insert_date		{Insert Date}
	count_char		{Count letters}
	countnum_all	{The number of letters}
	countnum_char	{The number of letters(excluding return)}
	countnum_line	{The number of lines}
	eval			{Eval text by Tcl/Tk}
	exec			{Exec text by Shell}
	del_intrp		{Stop the Interpreter...}
	wrap			Wrap
	topmost			{Always On Top}
	quit			Quit
	msg_stop_intrp	{Select a Slave Interpreter that you want to stop}
	msg_jump_page   {Select a Page to Jump}
	sort_asc		{To ascending order}
	sort_desc		{To descending order}
	sort_page		{Page number}
	sort_sum		Summary
	sort_dic		{Dictionary sort}
}




# ==========================================================
# Util
# ==========================================================
proc isInt {num} {
	if{[regexp {^\d+$} $num]} {
		return 1
	} else {
		return 0
	}
}


# if there is not selected string then return all text
proc getSelectedString {txt} {
	if {{}==[$txt tag ranges sel]} {
		set str [$txt get 1.0 "end-1 char"]
		if {{}==$str} {return {}} else {return $str}
	} else {
		return [$txt get sel.first sel.last]
	}
}

	
proc countChar {txt} {
	set str [getSelectedString $txt]
	set all  [string length $str]
	set line [expr 1+[regsub -all \n $str {} str]]
	set num [string length $str]
	return [list $all $num $line]
}
# ==========================================================
# File System Util
# ==========================================================
proc writeText { file str } {
	set temp temp.dat
	if { [file exists $temp] } {

	}
	if { [catch {open {temp.dat} w} f] } { error $f }
	puts -nonewline $f $str
	close $f
	if { [catch {file delete $file} msg] } {error $msg}
	if { [catch {file rename temp.dat $file} msg] } {error $msg}
}


proc readText { file } {
	if { [catch {open $file r} f] } {
		error $f
	}
	set str [read -nonewline $f]
	close $f
	return $str
}


#Return all files in dir
proc findFiles { dir } {
	if { ![file exists $dir] } { return }
	if { [file isfile $dir] } { return $dir }
	list flist
	set pattern [file join $dir *]
	foreach file [lsort [glob -nocomplain $pattern -type f]] {
		if { [file isfile $file] } { lappend flist $file } 
	}
	return $flist
}



# ==========================================================
# Read/Write Preference Info
# ==========================================================
proc writeSession {} {
	set str {}
	append str "[winfo geometry .]\n"
	append str "$::pref_win_topmost\n"
	append str "$::pref_page\n"
	append str $::pref_wrap
	writeText [file join $::pref_datadir $::pref_sessionfile] $str
}


proc readSession {} {
	set sessionfile [file join $::pref_datadir $::pref_sessionfile]
	if { ![file exists $sessionfile] } { return }
	set plist [split [readText $sessionfile] "\n"]
	for { set i 0 } { $i<[llength $plist] } { incr i } {
		set tmp [lindex $plist $i]
		switch $i {
			0 { if { [regexp {^\d+x\d+\+\d+\+\d+$} $tmp] } { set ::pref_win_geometry $tmp } }
			1 { if { 0==$tmp || 1==$tmp} { set ::pref_win_topmost  $tmp } }
			2 { if { 0<$tmp } { set ::pref_page $tmp } }
			3 { if { 0==$tmp || 1==$tmp } { set ::pref_wrap $tmp } }
			default {}
		}
	}
}


#if there is args1, set args1 as a working dir
proc getWorkingDir {} {
	if { 0 < $::argc } {
		set ::pref_datadir [lindex $::argv 0]
	}
}


#import resource file in working dir
proc readResource {} {
	set rcfile [file join $::pref_datadir $::pref_rcfile]
	if { ![file exists $rcfile] } { return }
	eval [readText $rcfile]
}


# ==========================================================
# Read/Write Page Text
# ==========================================================
proc writePage { page_number } {
	writeText [file join $::pref_datadir $page_number]  [.stext.text get 1.0 "end-1 char"]
	updateSummary $page_number [.stext.text get 1.0 1.end]
}


proc readPage { page_number } {
	if { ![file exists [file join $::pref_datadir $page_number]] } {
		.stext.text delete 1.0 end
		.stext.text insert 1.0 {}
		return
	}
	.stext.text delete 1.0 end
	.stext.text insert 1.0 [readText $page_number]
	.stext.text edit reset
}


# ==========================================================
# Eval and Exec
# ==========================================================



proc slavesExit {name} {
	interp delete $name
	if { [winfo exists .interplist] } { updateSlaveInfo }
}


proc evalText {} {
	set str [getSelectedString .stext.text]
	set child [interp create]
	interp alias $child exit {} slavesExit $child
	
	set ::slaveinfo($child) [list [clock seconds] [lindex [split $str \n] 0]]
	if { [winfo exists .interplist] } { updateSlaveInfo }

	$child eval { package require Tk }
	$child eval " wm title . $child "
	$child eval { wm protocol . WM_DELETE_WINDOW { exit }}
	$child eval $str
}



proc updateSlaveInfo {} {
	.interplist.f1.l delete 0 end
	foreach i [interp slaves {}] {
		set info $::slaveinfo($i)
		set sttime [clock format [lindex $info 0] -format {%Y-%m-%d %H:%M:%S}]
		set line   [lindex $info 1]
		.interplist.f1.l insert end "$i     $sttime     $line"
	}
}



proc deleteSlave {} {
	foreach index [.interplist.f1.l curselection] {
		set intr [lindex [.interplist.f1.l get $index] 0]
		interp delete $intr
		updateSlaveInfo
	}
}


proc createInterpList { uistring } {
	upvar $uistring uistr
	toplevel .interplist
	frame .interplist.f1
	frame .interplist.f2
	label     .interplist.f1.label -anchor w -text $uistr(msg_stop_intrp)
	listbox   .interplist.f1.l -yscrollcommand {.interplist.f1.y set}
	scrollbar .interplist.f1.y -command {.interplist.f1.l yview}
	button .interplist.f2.ok -text OK -command { deleteSlave }
	button .interplist.f2.cancel -text Cancel -command {wm withdraw .interplist}
	pack .interplist.f1.label -side top -fill x
	pack .interplist.f1.y -side right -fill y
	pack .interplist.f1.l -fill both -expand yes
	pack .interplist.f2.ok .interplist.f2.cancel -side left -expand yes -fill x
	pack .interplist.f2 -side bottom -fill x
	pack .interplist.f1 -side top -expand yes -fill both
	bind .interplist <Escape> {wm withdraw .interplist}
	bind .interplist <Double-1> { deleteSlave }

	set w 400
	set h 200
	wm geometry .interplist "$w\x$h+[expr ([winfo screenwidth .] - $w)/2]+[expr ([winfo screenheight .] - $h)/2]"
}



proc showInterpList {} {
	if { ![winfo exists .interplist] } {
		createInterpList $::UISTRING($::pref_lang)
	}
	updateSlaveInfo
	wm deiconify .interplist
	focus -force .interplist
}





proc execText {} {
	set str [getSelectedString .stext.text]
	set shell [file normalize $::pref_shell]
#	set f [open |$shell w]
#	puts $f $str
#	close $f

	set cmdlist [split $str "\n"]
	foreach cmd $cmdlist {
		if {$cmd!={}} {
		exec $::env(ComSpec) /c $cmd &
		}
	}
}








# ==========================================================
# Jump page
# ==========================================================


proc jumpPage { page_number } {
	if { 0==[regexp {^\d+$} $page_number] } {
		return
	}
	
	if { $page_number < 1 } {
		set page_number 1
	}
	# write
	writeSession
	writePage $::pref_page
	# read
	readPage $page_number
	# change value
	.page.num configure -text $page_number
	set ::pref_page $page_number
}



proc jumpNext {} { jumpPage [expr $::pref_page+1] }



proc jumpBack {} { jumpPage [expr $::pref_page-1] }



proc bind_JumpListUpDn {d} {
	set old [.jump.f2.list curselection]
	if { $old == {} } return
	set new [expr $old + $d]
	if {$new >= [.jump.f2.list index end]} {
		set new 0
	} elseif {$new < 0} {
		set new [expr [.jump.f2.list index end] - 1]
	}
	
	.jump.f2.list selection clear $old
	.jump.f2.list selection set $new
	
	set e [.jump.f2.list get $new]
	setJumpPageIndex [lindex $e 0]
}




proc createJumpList {uistring} {
	upvar $uistring uistr
	toplevel .jump
	label .jump.label -text $uistr(msg_jump_page) -anchor w
	frame .jump.f1
	frame .jump.f2
	entry .jump.f1.page
	button .jump.f1.ok -text Ok \
		-command {
			jumpPage [.jump.f1.page get];
			wm withdraw .jump
		}
	button .jump.f1.cancel -text Cancel -command {wm withdraw .jump}
	listbox .jump.f2.list -yscrollcommand { .jump.f2.y set }
	scrollbar .jump.f2.y -command {.jump.f2.list yview}
	pack .jump.f1.cancel .jump.f1.ok -side right -expand yes -fill x
	pack .jump.f1.page -fill y -expand yes
	pack .jump.f2.y -side right -fill y
	pack .jump.f2.list -fill both -expand yes
	pack .jump.label -side top -fill x
	pack .jump.f1 -side top -fill x  
	pack .jump.f2 -expand yes -fill both
	menu .jump.pop
	.jump.pop add radiobutton -label $uistr(sort_asc) -variable ::jumplist_sort(order) -value {-increasing} -command {updateJumpList}
	.jump.pop add radiobutton -label $uistr(sort_desc) -variable ::jumplist_sort(order) -value {-decreasing} -command {updateJumpList}
	.jump.pop add separator
	.jump.pop add radiobutton -label $uistr(sort_page) -variable ::jumplist_sort(key) -value 0 -command {updateJumpList}
	.jump.pop add radiobutton -label $uistr(sort_sum)  -variable ::jumplist_sort(key) -value 1 -command {updateJumpList}
	.jump.pop add separator
	.jump.pop add radiobutton -label $uistr(sort_dic) -variable ::jumplist_sort(type) -value {-dictionary} -command {updateJumpList}
	
	bind .jump.f2.list <3> {tk_popup .jump.pop %X %Y}
	bind .jump <Escape> {wm withdraw .jump}
	bind .jump.f1.page <Return> {
		jumpPage [.jump.f1.page get];
		wm withdraw .jump
	}
	bind .jump.f2.list <Double-1> {
		jumpPage [.jump.f1.page get];
		#wm withdraw .jump
	}
    bind .jump <MouseWheel> { .jump.f2.list yview scroll [expr {- (%D / 120) * 4}] units }
	bind .jump <Up>   { bind_JumpListUpDn -1 }
	bind .jump <Down> { bind_JumpListUpDn +1 }
	bind .jump <Control-Left>  { setJumpPageIndex [expr [.jump.f1.page get]-1] }
	bind .jump <Control-Right> { setJumpPageIndex [expr [.jump.f1.page get]+1] }
	bind .jump.f2.list <B1-ButtonRelease> {
		set index [.jump.f2.list curselection]
		set e [.jump.f2.list get $index]
		setJumpPageIndex [lindex $e 0]
	}
	set w 400
	set h 200
	wm geometry .jump "$w\x$h+[expr ([winfo screenwidth .] - $w)/2]+[expr ([winfo screenheight .] - $h)/2]"

	#Create summary
	writePage $::pref_page
	set flist [findFiles $::pref_datadir]
	foreach file $flist {
		set name [file tail $file]
		if { 0!=[regexp {^\d+$} $name] } {
			if {[catch {open $file r} f]} {
				set ::summary($name) {}
			} else {
				updateSummary $name [gets $f]
			}
			close $f
		}
	}
}


proc setJumpPageIndex {index} {
	if {1>$index} {set index 1}
	.jump.f1.page delete 0 end
	.jump.f1.page insert 0 $index
	.jump.f1.page selection range 0 end
	set i [lsearch -regexp [.jump.f2.list get 0 end] ^\\s+$index\\s]
	if {$i!=-1} {
		.jump.f2.list selection set $i
		.jump.f2.list see $i
	} else {
		.jump.f2.list selection clear 0 end
	}
}


proc updateJumpList {} {
	set llist {}
	set id [array startsearch ::summary]
	while {{}!=[set key [array nextelement ::summary $id]]} {
		lappend llist [list $key  $::summary($key)]
	}
	array donesearch ::summary $id
	#sort
	set llist [lsort $::jumplist_sort(order) $::jumplist_sort(type) -index $::jumplist_sort(key) $llist]
	#add
	.jump.f2.list delete 0 end
	foreach val $llist {
		set page [lindex $val 0]
		set str  [lindex $val 1]
		.jump.f2.list insert end [format {% 8u    %s} $page $str]
	}
}


proc showJumpList {} {
	writePage $::pref_page

	if { ![winfo exists .jump] } {
		createJumpList $::UISTRING($::pref_lang)
	}
	updateJumpList
	setJumpPageIndex $::pref_page
	wm deiconify .jump
	focus -force .jump.f1.page
}



# ==========================================================
# Exchange page and Create new page
# ==========================================================
proc exchangePage {source target} {
	if {$target < 1} {return}
	set src [file join $::pref_datadir $source]
	set tgt [file join $::pref_datadir $target]
	set tmp [file join $::pref_datadir $target.tmp]
	if { ![file exists $tgt] } {
		if { [catch { open $tgt w } f] } {error $f}
		close $f
	}
	if { [catch {file rename $tgt $tmp} msg ] } { error $msg }
	if { [catch {file rename $src $tgt} msg ] } { error $msg }
	if { [catch {file rename $tmp $src} msg ] } { error $msg }
	writeSession
	.page.num configure -text $target
	set ::pref_page $target
	
	set srcstr [readText $source]
	updateSummary $source [lindex [split $srcstr \n] 0]
	updateSummary $target [.stext.text get 1.0 1.end]

	.stext.text mark set insert end
}



proc exchangeBack {} {
	writePage $::pref_page
	exchangePage $::pref_page [expr $::pref_page - 1] 
}

	

proc exchangeNext {} {
	writePage $::pref_page
	exchangePage $::pref_page [expr $::pref_page + 1] 
}
# ==========================================================
# Command
# ==========================================================
proc updateSummary {page str} {
	if {$str=={}} {
		array unset ::summary $page
	} else {
		set ::summary($page) $str
	}
}
# Quit this Application
proc exitThis {} {
	writeSession
	writePage $::pref_page
	# Show Alert
	if { $::pref_quitAlert } {
		if { 1==[tk_dialog .dlg {Alert !!} \
				{ Do you really want to quit? } {} 1 \
				{   Quit  } { Cancel  }] } {
			return
		}
	}
	# Remove Empty File
	set flist [findFiles $::pref_datadir]
	foreach file $flist {
		if { ![file size $file] } {
			if { [catch {file delete $file} msg] } {
			error $msg
			}
		}
	}
	tcl_exit
}
rename exit tcl_exit
rename exitThis exit








proc insertDate {} {
	set now [clock seconds]
	#Replace %J to Japanease Weekday
	set weekday_jpn [lindex $::WEEKDAY_JPN [clock format $now -format %w]]
	regsub -all %J $::pref_clockformat $weekday_jpn newformat
	
	set date [clock format $now -format $newformat]
	.stext.text insert insert $date
}



proc countStextChar {} {
	upvar $::UISTRING($::pref_lang) uistr
	set nums [countChar .stext.text]
	set str {}
	append str [format "% 5d :  %s\n" [lindex $nums 0] $uistr(countnum_all)  ]
	append str [format "% 5d :  %s\n" [lindex $nums 1] $uistr(countnum_char) ]
	append str [format "% 5d :  %s\n" [lindex $nums 2] $uistr(countnum_line) ]
	tk_messageBox -message $str -title $uistr(count_char)
	#$uistr(copy)
}


proc changeFontOfNote {family size} {
	# font configure system    -family "\{$family\}" -size $size
	font configure font_note -family "\{$family\}" -size $size
}



proc doAutoSave {} {
	writeSession
	writePage $::pref_page
	after [expr $::pref_save_timesec * 1000] doAutoSave
}




proc changeWrap {} {
	set ::pref_wrap [expr $::pref_wrap^1]
	if { 0==$::pref_wrap } {
		.stext.text configure -wrap none
	} else {
		.stext.text configure -wrap word
	}
}



proc changeTopmost {} {
	set ::pref_win_topmost [expr $::pref_win_topmost^1]
	setTopmost . $::pref_win_topmost
	set title_add {}
	if {$::pref_win_topmost} { set title_add {  [top]} }
	wm title . "$::pref_win_name $title_add"
	focus -force .stext.text
}
proc setTopmost { w bool } {
	if { $::tcl_platform(platform)=={windows} } {
		raise $w
		wm attributes $w -topmost $bool
	} else {
		set taglist [bindtags $w]
		set index [lsearch $taglist topmost$w]
		if { $bool } {
			if {$index == -1} {
			bindtags $w [linsert $taglist 0 topmost$w]
			bind topmost$w <Visibility> "raise $w" 
			}
		} else {
			if {$index != -1} {
			bindtags $w [lreplace $taglist $index $index]
			bind $w <Visibility> {}
			}
		}
	}
}




proc init {uistring} {
	upvar $uistring uistr
	# import initfile & check var
	getWorkingDir
	if { [file exists $::pref_datadir] } {
		if { ![file isdirectory $::pref_datadir] } {
			error "$::pref_datadir is not directory"
		}
	} else {
		if { [catch {file mkdir $::pref_datadir} msg] } {
			error $msg
		}
	}
	set ::pref_datadir [file normalize $::pref_datadir]
	cd $::pref_datadir
	
	#
	# create main
	#
	frame .stext -bg $::pref_col_text_bg 
	scrollbar .stext.y -command {.stext.text yview}
	text  .stext.text -relief flat -yscrollcommand {.stext.y set} \
		-undo yes -tabs [expr $::pref_tabstop * [font measure font_note 0]]\
		-background $::pref_col_text_bg \
		-foreground $::pref_col_text_fg \
		-selectbackground $::pref_col_text_sbg \
		-selectforeground $::pref_col_text_sfg \
		-insertbackground $::pref_col_text_cur
	pack .stext.y -side right -fill y
	pack .stext.text -fill both -expand yes
	
	frame .page -bg $::pref_col_page_bg
	button .page.back -text "<<" -relief groove \
		-padx 1 -pady 1 \
		-background $::pref_col_btn_bg \
		-foreground $::pref_col_btn_fg \
		-activebackground $::pref_col_btn_bg \
		-activeforeground $::pref_col_btn_fg \
		-command { jumpBack }
	button .page.next -text ">>" -relief groove \
		-padx 1 -pady 1 \
		-background $::pref_col_btn_bg \
		-foreground $::pref_col_btn_fg \
		-activebackground $::pref_col_btn_bg \
		-activeforeground $::pref_col_btn_fg \
		-command { jumpNext }
	button .page.exit -text "  $uistr(quit)  " -relief groove \
		-padx 1 -pady 1 \
		-background $::pref_col_btn_bg \
		-foreground $::pref_col_btn_fg \
		-activebackground $::pref_col_btn_bg \
		-activeforeground $::pref_col_btn_fg \
		-command { exit }
	label .page.num -font font_note -text $::pref_page \
		-background $::pref_col_page_bg \
		-foreground $::pref_col_page_fg
	pack .page.back -side left
	pack .page.next -side left
	pack .page.exit -side right
	pack .page.num -fill both -expand yes
	
	pack .page -side bottom -fill x
	pack .stext -fill both -expand yes
	wm title . $::pref_win_name
	#
	# create popup
	#
	menu .pop ;#-tearoff no
	.pop add command -label $uistr(cut) -accelerator {Ctrl+X} -command { tk_textCut .stext.text };
	.pop add command -label $uistr(copy)  -accelerator {Ctrl+C} -command { tk_textCopy .stext.text };
	.pop add command -label $uistr(paste) -accelerator {Ctrl+V} -command { tk_textPaste .stext.text };
	.pop add command -label $uistr(selectall) -accelerator {Ctrl+A} -command { .stext.text tag add sel 1.0 end };
	.pop add separator
	.pop add command -label $uistr(insert_date) -accelerator {Ctrl+D} -command { insertDate }
	.pop add command -label $uistr(count_char) -command { countStextChar }
	.pop add separator
	.pop add command -label $uistr(back) -accelerator {Ctrl+Left}  -command { jumpBack }
	.pop add command -label $uistr(next) -accelerator {Ctrl+Right} -command { jumpNext }
	.pop add command -label $uistr(exchange_back) -accelerator {Ctrl+Alt+Left}  -command { exchangeBack }
	.pop add command -label $uistr(exchange_next) -accelerator {Ctrl+Alt+Right} -command { exchangeNext }
	.pop add command -label $uistr(go) -accelerator {Ctrl+G} -command {showJumpList}
	.pop add separator
	.pop add command -label $uistr(eval) -accelerator {F5} -command { evalText }
	.pop add command -label $uistr(exec) -accelerator {Shift+F5} -command { execText }
	.pop add command -label $uistr(del_intrp) -accelerator {F6} -command { showInterpList }


	.pop add separator
	.pop add checkbutton -label $uistr(wrap) -accelerator {F11} -variable ::pref_wrap \
		-command {
		set ::pref_wrap [expr $::pref_wrap^1]
		changeWrap
	}
	.pop add checkbutton -label $uistr(topmost) -accelerator {F12} -variable ::pref_win_topmost \
		-command {
		set ::pref_win_topmost [expr $::pref_win_topmost^1]
		changeTopmost
		}
 	
	.pop add separator
	.pop add command -label $uistr(quit) -accelerator {Ctrl+Q} -command { exit }
	#
	# bind event
	#
	wm protocol . WM_DELETE_WINDOW { exit }
	bind .stext.text <3> {tk_popup .pop %X %Y}
	bind . <Control-q> { exit }
	bind . <F1> {};# help
	bind . <Control-a> { .stext.text tag add sel 1.0 end }
	bind . <Control-Left> { jumpBack }
	bind . <Control-Right> { jumpNext }
	bind . <Control-Alt-Left>  { exchangeBack }
	bind . <Control-Alt-Right> { exchangeNext }
	bind . <Control-g> { showJumpList }
	bind .page.num <Double-1> {showJumpList}
	bind . <Control-d> { insertDate }
	bind . <F5> { evalText }
	bind . <Shift-F5> {execText }
	bind . <F6>  { showInterpList }
	bind . <F11> { changeWrap }
	bind . <F12> { changeTopmost }
	#
	# read preference file and init widget
	#
	readSession
	.page.num configure -text $::pref_page
	.stext.text configure -font font_note
	readPage $::pref_page

	wm geometry	. $::pref_win_geometry
}



# ==========================================================
# start
# ==========================================================
readResource

init $::UISTRING($::pref_lang)
if { 0<$::pref_save_timesec } { doAutoSave }
	set ::pref_win_topmost [expr $::pref_win_topmost^1];
	changeTopmost
	set ::pref_wrap [expr $::pref_wrap^1]
	changeWrap
focus -force .stext.text

